//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     parse.h
// Purpose:    Syntax Analysis
// Last Change:12.09.2002
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2000-2002 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	PARSE_INCLUDED
#define PARSE_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef BOOL_INCLUDED
#include "bool.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef OUT_INCLUDED
#include "out.h"
#endif

#ifndef ATOM_INCLUDED
#include "atom.h"
#endif

#ifndef VAR_INCLUDED
#include "var.h"
#endif

#ifndef TERM_INCLUDED
#include "term.h"
#endif

#ifndef TLIST_INCLUDED
#include "tlist.h"
#endif

#ifndef LIT_INCLUDED
#include "lit.h"
#endif

#ifndef FML_INCLUDED
#include "fml.h"
#endif

#ifndef FLIST_INCLUDED
#include "flist.h"
#endif

#ifndef TOK_INCLUDED
#include "tok.h"
#endif

#ifndef LEX_INCLUDED
#include "lex.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// PARSE_STACKSIZE: Size of the Operator and Operand Stacks.
//-----------------------------------------------------------------------------

static const int PARSE_STACKSIZE = 100;

//-----------------------------------------------------------------------------
// PARSE_MAX_ERRS: Maximal number of errors before parsing is aborted.
//-----------------------------------------------------------------------------

static const int PARSE_MAX_ERRS = 10;

//=============================================================================
// Class for Currently Parsed File (has only class methods, no instances):
//=============================================================================

class parse_c {
	public:

//-----------------------------------------------------------------------------
// Static Member Functions (Class Methods):
//-----------------------------------------------------------------------------

	// Parse a file:
	static flist_t	parse(in_c* input);

//-----------------------------------------------------------------------------
// Static Data Members (Class Attributes):
//-----------------------------------------------------------------------------

	private:

	// Lex: The input token stream for this file (interface to scanner).
	static lex_c*	Lex;

	// Tok: The type of the current token (i.e. Lex->tok_type()).
	static tok_t	Tok;

	// NumErrs: Number of syntax errors discovered so far.
	static int	NumErrs;

	// OpStack: Operator Stack.
	static op_c	*OpStack[PARSE_STACKSIZE];

	// OpStackPtr: Stackpointer for operator stack (one above top element).
	static op_c	**OpStackPtr;

	// OpStackEnd: One above last possible element of operator stack.
	static op_c	**OpStackEnd;

	// FmlStack: Operand (Formula) Stack.
	static fml_c	*FmlStack[PARSE_STACKSIZE];

	// FmlStackPtr: Stackpointer for operand stack (one above top element).
	static fml_c	**FmlStackPtr;

	// FmlStackEnd: One above last possible element of operand stack.
	static fml_c	**FmlStackEnd;

	// LogOpWithinNot: AND or OR were used inside default negation.
#ifdef VER_SEM_STABLE
	static bool_t	LogOpWithinNot;
#endif

//-----------------------------------------------------------------------------
// Auxillary Functions:
//-----------------------------------------------------------------------------

	private:

	// error: Print error message for current token.
	static void error(out_syn_t msg);

	// error_head: Print error message for invalid '_' or 'not' in the head
	static void error_head(fml_t fml);

	// error_var: Print error message for unbound variable.
	static void error_var(void);

	// next: Switch to next token.
	static inline void next()
		{ Lex->next();
		  Tok = Lex->tok_type(); }

	// parse_pred: Parse a predicate.
	static atom_c *parse_pred(void);

	// parse_term: Parse a term.
	static term_c *parse_term(void);

	// parse_args: Parse arguments of a literal (list of terms).
	static tlist_c *parse_args(void);

	// parse_lit: Parse a literal.
	static lit_c *parse_lit(bool_t parenthesized);

	// within_defneg: We currently reading the argument of a not operator.
	static bool_t within_defneg();

	// pop_ops: Pop operators from the stack until specified priority.
	static void pop_ops(int prio);

	// push_op: Push an operator. Pop previous ones if necessary.
	static bool_t push_op(op_c *op);

	// push_fml: Push an operand (i.e. a formula).
	static bool_t push_fml(fml_c *fml);

	// pop_fml: Return the final formula.
	static fml_c *pop_fml(void);

	// parse_fml: Parse a formula (auxillary function).
	static fml_c *parse_fml(bool_t parenthesized);

	// parse_query: Parse a query, translate into formula about $answer.
	static fml_t parse_query(void);

	// parse_fml_or_query: Parse formula or query, store as clauses.
	static fml_t parse_fml_or_query(void);

	// check_head: Check whether '_' or 'not' are used in positive context.
	static bool_t check_head(fml_t fml, int sign);

	// parse_cmd: Parse and process a command.
	static bool_t parse_cmd(void);

//-----------------------------------------------------------------------------
// End of Class:
//-----------------------------------------------------------------------------

};

//=============================================================================
// End of Include File:
//=============================================================================

#endif

